--[[
# Copyright 2001-2014 Cisco Systems, Inc. and/or its affiliates. All rights
# reserved.
#
# This file contains proprietary Detector Content created by Cisco Systems,
# Inc. or its affiliates ("Cisco") and is distributed under the GNU General
# Public License, v2 (the "GPL").  This file may also include Detector Content
# contributed by third parties. Third party contributors are identified in the
# "authors" file.  The Detector Content created by Cisco is owned by, and
# remains the property of, Cisco.  Detector Content from third party
# contributors is owned by, and remains the property of, such third parties and
# is distributed under the GPL.  The term "Detector Content" means specifically
# formulated patterns and logic to identify applications based on network
# traffic characteristics, comprised of instructions in source code or object
# code form (including the structure, sequence, organization, and syntax
# thereof), and all documentation related thereto that have been officially
# approved by Cisco.  Modifications are considered part of the Detector
# Content.
--]]
--[[
detection_name: NetBIOS-ns
version: 2
description: Netbios nameservice.
bundle_description: $VAR1 = {
          'NetBIOS-ns' => 'Netbios nameservice.'
        };

--]]

require "DetectorCommon"



local DC = DetectorCommon
local FT = flowTrackerModule

gDetector = nil

DetectorPackageInfo = {
    name =  "NetBIOS-ns",
    proto =  DC.ipproto.udp,
    client = {
        init =  'client_init',
        clean = 'client_clean',
		validate =  'client_validate',
		minimum_matches = 1
    }
}

gSfAppIdNBNS = 754

gPatterns = {
    nb_nbin =  { '\000\032\000\001', 46, gSfAppIdNBNS},
    nb_nstat = { '\000\033\000\001', 46, gSfAppIdNBNS},
    nb_reg =   { '\041\016', 2, gSfAppIdNBNS},
    nb_quer =  { '\001\016', 2, gSfAppIdNBNS},
    nb_quer2 = { '\000\000', 2, gSfAppIdNBNS},   
}

gFastPatterns = {
    {DC.ipproto.udp, gPatterns.nb_nbin},
    {DC.ipproto.udp, gPatterns.nb_nstat},
}

gAppRegistry = {
	--AppIdValue          Extracts Info
	---------------------------------------
	{gSfAppIdNBNS,		         0}
}

flowTrackerTable = {}

function clientInProcess(context)

	DC.printf('%s: Inprocess Client, packetCount: %d\n', DetectorPackageInfo.name, context.packetCount)
	return DC.clientStatus.inProcess
end

function clientSuccess(context)
    context.detectorFlow:setFlowFlag(DC.flowFlags.clientAppDetected)
    DC.printf('%s: Detected Client, packetCount: %d\n', DetectorPackageInfo.name, context.packetCount)
    gDetector:client_addApp(appServiceId, appTypeId, appProductId, "", gSfAppIdNBNS);
    flowTrackerTable[context.flowKey] = Nil
    return DC.clientStatus.success
end

function clientFail(context)
    DC.printf('%s: Failed Client, packetCount: %d\n', DetectorPackageInfo.name, context.packetCount)
    flowTrackerTable[context.flowKey] = Nil
    return DC.clientStatus.einvalid
end

--[[ Core engine calls DetectorInit() to initialize a detector.
--]]
function client_init( detectorInstance, configOptions)
    gDetector = detectorInstance
    DC.printf ('%s:DetectorInit()\n', DetectorPackageInfo.name);
	gDetector:client_init()
	appTypeId = 23
	appProductId = 513
	appServiceId = 16
	--DC.printf ('%s:DetectorValidator(): appTypeId %d, product %d, service %d\n', DetectorPackageInfo.name, appTypeId, appProductId, appServiceId)

    --register pattern based detection
    for i,v in ipairs(gFastPatterns) do
        if ( gDetector:client_registerPattern(v[1], v[2][1], #v[2][1], v[2][2], v[2][3]) ~= 0) then
            DC.printf ('%s: register pattern failed for %s\n', DetectorPackageInfo.name,v[2][1])
        else
            DC.printf ('%s: register pattern successful for %s\n', DetectorPackageInfo.name,v[2][1])
        end
    end

	for i,v in ipairs(gAppRegistry) do
		pcall(function () gDetector:registerAppId(v[1],v[2]) end)
	end

    return gDetector
end

--[[Validator function registered in DetectorInit()
--]]
function client_validate()
    local context = {}
    context.detectorFlow = gDetector:getFlow()
	context.packetCount = gDetector:getPktCount()
    context.packetDataLen = gDetector:getPacketSize()
    context.packetDir = gDetector:getPacketDir()
	context.flowKey = context.detectorFlow:getFlowKey()
    context.srcPort = gDetector:getPktSrcPort()
    context.dstPort = gDetector:getPktDstPort()
    local size = context.packetDataLen
    local dir = context.packetDir
    local srcPort = context.srcPort
    local dstPort = context.dstPort
    local flowKey = context.flowKey

    DC.printf ('client NetBIOS-ns: packetCount %d, dir %d, size %d\n', context.packetCount, dir, size);

    if (dir == 0 and size > 49 and dstPort == 137) then
        DC.printf ('client NetBIOS-ns: checking patterns\n')
        
        if ((gDetector:memcmp(gPatterns.nb_nbin[1], #gPatterns.nb_nbin[1], gPatterns.nb_nbin[2]) == 0) and 
            ((gDetector:memcmp(gPatterns.nb_reg[1], #gPatterns.nb_reg[1], gPatterns.nb_reg[2]) == 0) or 
             (gDetector:memcmp(gPatterns.nb_quer[1], #gPatterns.nb_quer[1], gPatterns.nb_quer[2]) == 0)) ) then

            DC.printf ('client NetBIOS-ns: NB name query\n')
            return clientSuccess(context)

        elseif ((gDetector:memcmp(gPatterns.nb_nstat[1], #gPatterns.nb_nstat[1], gPatterns.nb_nstat[2]) == 0) and 
            (gDetector:memcmp(gPatterns.nb_quer2[1], #gPatterns.nb_quer2[1], gPatterns.nb_quer2[2]) == 0)) then

            DC.printf ('client NetBIOS-ns: NB nstat query\n')
            return clientSuccess(context)
            
        end 

    end

    return clientFail(context)
end

function client_clean()
end

